module AssetsLedger
  class Ledger
    attr_reader :js_dependencies, :css_dependencies

    def initialize
      @js  = Set.new
      @css = Set.new
    end

    # Add JS to ledger
    def add_js(*names)
      @js.merge(dependencies_for(names, js_libs))
    end

    # Add CSS to ledger
    def add_css(*names)
      @css.merge(dependencies_for(names, css_libs))
    end

    # JS dependency list
    # @return [Array] of JS dependencies.
    def js_dependencies
      @js.to_a
    end

    # CSS dependency list
    # @return [Array] of CSS dependencies.
    def css_dependencies
      @css.to_a
    end

    # Returns a string with the source path for the given JS lib
    # @return [String] JS source path.
    # @param [Symbol] lib identifier
    # @example Get JS source path for jQuery
    #   assets_ledger.js_source_for(:jquery)
    def js_source_for(lib)
      js_libs[lib]
    end

    protected

    def dependencies_for(names, assets)
      names.flatten.flat_map { |name| Array(assets[name] || name) }
    end

    def js_libs
      {}
    end

    def css_libs
      {}
    end
  end
end
